Package: robustlmm
Type: Package
Title: Robust Linear Mixed Effects Models
Version: 3.4-5
Date: 2026-06-21
Author: Manuel Koller [aut, cre]
Authors@R: person(given = "Manuel", family = "Koller", role = c("aut", "cre"), email = "kollerma@proton.me")
Maintainer: Manuel Koller <kollerma@proton.me>
Description: Implements the Robust Scoring Equations estimator to fit linear
    mixed effects models robustly.
    Robustness is achieved by modification of the scoring equations
    combined with the Design Adaptive Scale approach.
License: GPL-2
URL: https://github.com/kollerma/robustlmm
LazyLoad: yes
Depends: lme4 (>= 2.0-1), Matrix (>= 1.6-2), R (>= 3.5.0)
Suggests: ggplot2, reshape2, microbenchmark, emmeans (>= 1.4),
        estimability, lqmm, MASS, lemon, RColorBrewer, skewt, fs,
        dplyr, ggh4x, testthat, robustvarComp, confintROB
Imports: lattice, nlme, methods, robustbase (>= 0.93), xtable, Rcpp (>=
        0.12.2), fastGHQuad, numDeriv, parallel, rlang, utils,
        reformulas
Collate: 'ghq.R' 'psiFunc2.R' 'redescendingPsi.R' 'AllClass.R'
        'rlmer.R' 'accessors.R' 'fromLme4.R' 'DAS-scale.R'
        'fit.effects.R' 'helpers.R' 'AllGeneric.R' 'lmer.R'
        'mutators.R' 'plot.R' 'generateAnovaDatasets.R'
        'generateLongitudinalDatasets.R' 'plotLongitudinalBySubject.R'
        'generateMixedEffectDatasets.R'
        'generateSensitivityCurveDatasets.R' 'manageDatasets.R'
        'ransac.R' 'influence.R' 'influence_full.R' 'bootstrapWald.R'
        'fitDatasets.R' 'processFit.R' 'processFile.R'
        'simulationStudies.R' 'asymptoticEfficiency.R' 'emmeans.R'
LinkingTo: Rcpp, robustbase, Matrix
Encoding: UTF-8
RcppModules: psi_function_module
RoxygenNote: 7.3.3
Config/build/clean-inst-doc: false
NeedsCompilation: yes
Packaged: 2026-06-21 09:44:49 UTC; kollerma
Repository: CRAN
Date/Publication: 2026-06-21 10:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-21 11:24:15 UTC; unix
Archs: robustlmm.so.dSYM
