useDynLib(robustlmm)
export(compare)
export(cPsi)
export(getInfo)
export(getME)
export(huberPsiRcpp)
export(lmerNoFit)
export(plot.rlmerMod)
export(rlmer)
export(caseweightIF)
export(vcov_sandwich)
export(implicitIF_full)
export(smoothPsi)
export(theta)
export(generateAnovaDatasets)
export(generateLongitudinalDatasets)
export(plotLongitudinalBySubject)
export(prepareMixedEffectDataset)
export(generateMixedEffectDatasets)
export(generateSensitivityCurveDatasets)
export(bindDatasets)
export(splitDatasets)
export(createDatasetsFromList)
export(saveDatasets)
export(lapplyDatasets)
export(fitDatasets_lmer)
export(fitDatasets_lmer_bobyqa)
export(fitDatasets_lmer_Nelder_Mead)
export(fitDatasets_rlmer)
export(createRhoFunction)
export(extractTuningParameter)
export(extractPredefinedTuningParameter)
export(fitDatasets_rlmer_DAStau)
export(fitDatasets_rlmer_DAStau_lmerNoFit)
export(fitDatasets_rlmer_DASvar)
export(fitDatasets_rlmer_DAStau_noAdj)
export(fitDatasets_rlmer_DAStau_k_0_5)
export(fitDatasets_rlmer_DAStau_k_0_5_noAdj)
export(fitDatasets_rlmer_DAStau_k_2)
export(fitDatasets_rlmer_DAStau_k_2_noAdj)
export(fitDatasets_rlmer_DAStau_k_5)
export(fitDatasets_rlmer_DAStau_k_5_noAdj)
export(fitDatasets_rlmer_DAStau_bisq)
export(fitDatasets_rlmer_DAStau_sizeOBR)
export(fitDatasets_rlmer_ransac)
export(fitDatasets_rlmer_ransac_bisq)
export(makeBisquarePsi)
export(bisquarePsi)
export(ransac_lme4)
export(rlmer_ransac)
export(fitDatasets_heavyLme)
export(fitDatasets_lqmm)
export(fitDatasets_rlme)
export(fitDatasets_varComprob)
export(fitDatasets_varComprob_compositeTau)
export(fitDatasets_varComprob_compositeTau_OGK)
export(fitDatasets_varComprob_compositeTau_2SGS)
export(fitDatasets_varComprob_compositeS)
export(fitDatasets_varComprob_compositeS_OGK)
export(fitDatasets_varComprob_compositeS_2SGS)
export(fitDatasets_varComprob_S)
export(fitDatasets_varComprob_S_OGK)
export(fitDatasets_varComprob_S_2SGS)
export(shortenLabelsKS2022)
export(processFit)
export(mergeProcessedFits)
export(processFile)
export(processDatasetsInParallel)
export(loadAndMergePartialResults)
export(viewCopyOfSimulationStudy)
export(asymptoticVariance)
export(partialMoment_standardNormal)
export(asymptoticEfficiency)
export(findTuningParameter)
exportClasses(rlmerMod)
exportMethods(chgDefaults)
exportMethods(plot)
exportMethods(psi2propII)
exportMethods(show)
importFrom(fastGHQuad,gaussHermiteData)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,plot)
importFrom(grDevices, devAskNewPage)
importFrom(lattice,dotplot)
importFrom(lattice,qqmath)
importFrom(reformulas,findbars)
importFrom(lme4,getME)
importFrom(lme4,isGLMM)
importFrom(lme4,isLMM)
importFrom(lme4,isNLMM)
importFrom(lme4,isREML)
importFrom(lme4,lmer)
importFrom(lme4,lmerControl)
importFrom(lme4,mkMerMod)
importFrom(reformulas,mkReTrms)
importFrom(reformulas,nobars)
importFrom(lme4,REMLcrit)
## S3 generic for sigma():  exported from 'stats' for R >= 3.3.0; from 'lme4' for R <= 3.2.x
if(getRversion() >= "3.3.0") {
    importFrom(stats, sigma)
    importFrom(stats, fivenum)
} else {
   importFrom(lme4, sigma)
}
importFrom(Matrix,.bdiag)
importFrom(Matrix,Diagonal)
importFrom(Matrix,forceSymmetric)
importFrom(Matrix,isDiagonal)
importFrom(Matrix,Matrix)
importFrom(Matrix,Matrix.Version)
importFrom(Matrix,sparseMatrix)
import(methods, Rcpp)
importFrom(Matrix,symmpart)
importFrom(methods,new)
importFrom(methods,setAs)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,setRefClass)
importFrom(methods, as)
importFrom(methods, "as<-")
importFrom(methods, coerce)
importFrom(methods, "coerce<-")
importFrom(methods, is)
importFrom(methods, slot)
importFrom(methods, "slot<-")
importFrom(methods, validObject)
importFrom(methods, show)
importFrom(nlme,fixef)
importFrom(nlme,ranef)
importFrom(nlme,VarCorr)
importFrom(rlang,warn)
importFrom(robustbase,summarizeRobWeights)
importFrom(stats,coef)
importFrom(stats, dchisq)
importFrom(stats, integrate)
importFrom(stats, cov2cor)
importFrom(stats, as.formula)
importFrom(stats, asOneSidedFormula)
importFrom(stats, delete.response)
importFrom(stats, deviance)
importFrom(stats, dnorm)
importFrom(stats, gaussian)
importFrom(stats, model.offset)
importFrom(stats, model.response)
importFrom(stats, model.weights)
importFrom(stats, na.pass)
importFrom(stats, napredict)
importFrom(stats, naresid)
importFrom(stats, ppoints)
importFrom(stats, printCoefmat)
importFrom(stats, qnorm)
importFrom(stats, quantile)
importFrom(stats, reorder)
importFrom(stats, setNames)
importFrom(stats, symnum)
importFrom(stats, terms.formula)
importFrom(stats, uniroot)
importFrom(stats, update.formula)
importFrom(stats,extractAIC)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,nobs)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,terms)
importFrom(stats,vcov)
importFrom(stats,cooks.distance)
importFrom(stats,influence)
importFrom(stats,confint)
importFrom(stats,weights)
importFrom(utils,as.roman)
importFrom(utils,getS3method)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,sessionInfo)
importFrom(utils,stack)
importFrom(utils,toLatex)
importFrom(xtable,print.xtable)
importFrom(xtable,xtable)
importMethodsFrom(Matrix,as.vector)
importMethodsFrom(Matrix,chol)
importMethodsFrom(Matrix,colMeans)
importMethodsFrom(Matrix,colSums)
importMethodsFrom(Matrix,crossprod)
importMethodsFrom(Matrix,determinant)
importMethodsFrom(Matrix,diag)
importMethodsFrom(Matrix,drop)
importMethodsFrom(Matrix,isDiagonal)
importMethodsFrom(Matrix,isTriangular)
importMethodsFrom(Matrix,rowMeans)
importMethodsFrom(Matrix,rowSums)
importMethodsFrom(Matrix,solve)
importMethodsFrom(Matrix,t)
importMethodsFrom(Matrix,tcrossprod)
importMethodsFrom(Matrix,which)
importMethodsFrom(robustbase,chgDefaults)
importMethodsFrom(robustbase,plot)
importFrom(robustbase,psiFunc)
importClassesFrom(robustbase,psi_func)
importClassesFrom(Rcpp, Module)
S3method(as.data.frame,VarCorr.rlmerMod)
S3method(coef,rlmerMod)
S3method(confint,rlmerMod)
S3method(deviance,rlmerMod)
S3method(dotplot,coef.rlmerMod)
S3method(dotplot,ranef.rlmerMod)
S3method(extractAIC,rlmerMod)
S3method(family,rlmerMod)
S3method(fitted,rlmerMod)
S3method(fixef,rlmerMod)
S3method(formula,rlmerMod)
S3method(getInfo,lmerMod)
S3method(getInfo,rlmerMod)
S3method(getME,rlmerMod)
S3method(isGLMM,rlmerMod)
S3method(isLMM,rlmerMod)
S3method(isNLMM,rlmerMod)
S3method(isREML,rlmerMod)
S3method(logLik,rlmerMod)
S3method(model.frame,rlmerMod)
S3method(model.matrix,rlmerMod)
S3method(nobs,rlmerMod)
S3method(plot,coef.rlmerMod)
S3method(plot,ranef.rlmerMod)
S3method(plot,rlmerMod)
S3method(predict,rlmerMod)
S3method(print,comparison.table)
S3method(print,rlmerMod_plots)
S3method(print,rlmerMod)
S3method(print,summary.rlmerMod)
S3method(print,VarCorr.rlmerMod)
S3method(print,xtable.comparison.table)
S3method(qqmath,ranef.rlmerMod)
S3method(ranef,rlmerMod)
S3method(residuals,rlmerMod)
S3method(sigma,rlmerMod)
S3method(summary,rlmerMod)
S3method(terms,rlmerMod)
S3method(update,rlmerMod)
S3method(VarCorr,rlmerMod)
S3method(VarCorr,summary.rlmerMod)
S3method(vcov,rlmerMod)
S3method(vcov,summary.rlmerMod)
S3method(cooks.distance,rlmerMod)
S3method(influence,rlmerMod)
S3method(weights,rlmerMod)
S3method(xtable,comparison.table)
S3method(processFit,"try-error")
S3method(processFit,"package-not-installed")
S3method(processFit,lmerMod)
S3method(processFit,rlmerMod)
S3method(processFit,heavyLme)
S3method(processFit,lqmm)
S3method(processFit,rlme)
S3method(processFit,varComprob)
